@@ -45,6 +45,7 @@
 #include <libgnomeui/gnome-file-entry.h>
 #include <libgnomeui/gnome-icon-list.h>
 #include <libgnomeui/gnome-icon-sel.h>
+#include <bonobo-activation/bonobo-activation.h>
 #include <limits.h>
 #include <signal.h>
 #include <stdio.h>
@@ -55,9 +56,9 @@
 
 /* structure for the icon selection dialog */
 struct IconSelectionData {
+	GtkWidget *dialog;
         GtkWidget *icon_selection;
 	GtkWidget *file_entry;
-	GtkWidget *top_level_window;
 	GtkWindow *owning_window;
 	gboolean dismissed;
 	EelIconSelectionFunction selection_function;
@@ -66,32 +67,6 @@ struct IconSelectionData {
 
 typedef struct IconSelectionData IconSelectionData;
 
-ArtIRect
-eel_gnome_canvas_world_to_window_rectangle (GnomeCanvas *canvas,
-					    ArtDRect world_rect)
-{
-	double x0, y0, x1, y1;
-	ArtIRect window_rect;
-
-	g_return_val_if_fail (GNOME_IS_CANVAS (canvas), eel_art_irect_empty);
-
-	gnome_canvas_world_to_window (canvas,
-				      world_rect.x0,
-				      world_rect.y0,
-				      &x0, &y0);
-	gnome_canvas_world_to_window (canvas,
-				      world_rect.x1,
-				      world_rect.y1,
-				      &x1, &y1);
-
-	window_rect.x0 = x0;
-	window_rect.y0 = y0;
-	window_rect.x1 = x1;
-	window_rect.y1 = y1;
-
-	return window_rect;
-}
-
 ArtIRect
 eel_gnome_canvas_world_to_canvas_rectangle (GnomeCanvas *canvas,
 					    ArtDRect world_rect)
@@ -592,7 +567,7 @@ get_terminal_command_prefix (gboolean for_command)
 	char *command;
 	guint i;
 	static const char *const commands[][3] = {
-		{ "gnome-terminal", "--start-factory-server --use-factory -x", "--login" },
+		{ "gnome-terminal", "-x",                                      "" },
 		{ "dtterm",         "-e",                                      "-ls" },
 		{ "nxterm",         "-e",                                      "-ls" },
 		{ "color-xterm",    "-e",                                      "-ls" },
@@ -671,25 +646,26 @@ eel_gnome_open_terminal (const char *command)
 }
 
 /* create a new icon selection dialog */
-
 static gboolean
 widget_destroy_callback (gpointer callback_data)
 {
-	IconSelectionData *selection_data = (IconSelectionData*) callback_data;
-	gtk_widget_destroy (selection_data->top_level_window);
+	IconSelectionData *selection_data;
+
+	selection_data = (IconSelectionData *) callback_data;
+	gtk_widget_destroy (selection_data->dialog);
 	g_free (selection_data);	
 	return FALSE;
 }
 
 /* set the image of the file object to the selected file */
 static void
-icon_selected_callback (GtkWidget *dialog, IconSelectionData *selection_data)
+icon_selected (IconSelectionData *selection_data)
 {
-	const gchar *icon_path;
+	const char *icon_path;
 	struct stat buf;
 	GtkWidget *entry;
 	
-	gnome_icon_selection_stop_loading ( GNOME_ICON_SELECTION (selection_data->icon_selection));
+	gnome_icon_selection_stop_loading (GNOME_ICON_SELECTION (selection_data->icon_selection));
 
 	entry = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (selection_data->file_entry));
 	icon_path = gtk_entry_get_text (GTK_ENTRY (entry));
@@ -707,59 +683,55 @@ icon_selected_callback (GtkWidget *dialog, IconSelectionData *selection_data)
 		selection_data->selection_function (icon_path, selection_data->callback_data);
 	}
 		
-	/* we have to get rid of the icon selection dialog, but we can't do it now, since the
-	 * file entry might still need it.  Do it when the next idle rolls around
+	/* We have to get rid of the icon selection dialog, but we can't do it now, since the
+	 * file entry might still need it. Do it when the next idle rolls around
 	 */ 
 	selection_data->dismissed = TRUE;
-	selection_data->top_level_window = dialog;
-	gtk_idle_add ((GtkFunction) widget_destroy_callback, selection_data);
+	gtk_idle_add (widget_destroy_callback, selection_data);
 }
 
 /* handle the cancel button being pressed */
 static void
-icon_cancel_pressed (GtkWidget *dialog, IconSelectionData *selection_data)
+icon_cancel_pressed (IconSelectionData *selection_data)
 {
 	/* nothing to do if it's already been dismissed */
 	if (selection_data->dismissed) {
 		return;
 	}
 	
-	gnome_icon_selection_stop_loading(GNOME_ICON_SELECTION (selection_data->icon_selection));	
-	/* remove pending idle routine, if necessary */
+	gtk_widget_destroy (selection_data->dialog);
 	g_free (selection_data);
-	gtk_widget_destroy (dialog);
 }
 
 /* handle an icon being selected by updating the file entry */
 static void
-list_icon_selected_callback (GnomeIconList *gil, gint num, GdkEvent *event, IconSelectionData *selection_data)
+list_icon_selected_callback (GnomeIconList *gil, int num, GdkEvent *event, IconSelectionData *selection_data)
 {
-	const gchar *icon;
+	const char *icon;
 	GtkWidget *entry;
 	
 	icon = gnome_icon_selection_get_icon (GNOME_ICON_SELECTION (selection_data->icon_selection), TRUE);
-
 	if (icon != NULL) {
 		entry = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (selection_data->file_entry));
-		gtk_entry_set_text(GTK_ENTRY (entry), icon);
+		gtk_entry_set_text (GTK_ENTRY (entry), icon);
 	}
 
 	/* handle double-clicks as if the user pressed OK */
-	if(event && event->type == GDK_2BUTTON_PRESS && ((GdkEventButton *)event)->button == 1) {
-		icon_selected_callback (selection_data->file_entry, selection_data);
+	if (event != NULL && event->type == GDK_2BUTTON_PRESS && ((GdkEventButton *) event)->button == 1) {
+		icon_selected (selection_data);
 	}
 }
 
 static void
-dialog_response_callback (GtkWidget *dialog, gint response_id, IconSelectionData *selection_data)
+dialog_response_callback (GtkWidget *dialog, int response_id, IconSelectionData *selection_data)
 {
 	switch (response_id) {
 	case GTK_RESPONSE_OK:
-		icon_selected_callback (dialog, selection_data);
+		icon_selected (selection_data);
 		break;
 	case GTK_RESPONSE_CANCEL:
 	case GTK_RESPONSE_DELETE_EVENT:
-		icon_cancel_pressed (dialog, selection_data);
+		icon_cancel_pressed (selection_data);
 		break;
 	default:
 		break;
@@ -768,26 +740,25 @@ dialog_response_callback (GtkWidget *dialog, gint response_id, IconSelectionData
 
 /* handle the file entry changing */
 static void
-entry_activated (GtkWidget *widget, IconSelectionData *selection_data)
+entry_activated_callback (GtkWidget *widget, IconSelectionData *selection_data)
 {
 	struct stat buf;
-	GtkWidget *icon_selection;
-	const gchar *filename;
+	GnomeIconSelection *icon_selection;
+	const char *filename;
 
 	filename = gtk_entry_get_text (GTK_ENTRY (widget));
-	if (!filename) {
+	if (filename == NULL) {
 		return;
 	}
 	
-	stat (filename, &buf);
-	if (S_ISDIR (buf.st_mode)) {
-		icon_selection = selection_data->icon_selection;
-		gnome_icon_selection_clear (GNOME_ICON_SELECTION (icon_selection), TRUE);
-		gnome_icon_selection_add_directory (GNOME_ICON_SELECTION (icon_selection), filename);
-		gnome_icon_selection_show_icons(GNOME_ICON_SELECTION (icon_selection));
+	if (stat (filename, &buf) == 0 && S_ISDIR (buf.st_mode)) {
+		icon_selection = GNOME_ICON_SELECTION (selection_data->icon_selection);
+		gnome_icon_selection_clear (icon_selection, TRUE);
+		gnome_icon_selection_add_directory (icon_selection, filename);
+		gnome_icon_selection_show_icons(icon_selection);
 	} else {
 		/* We pretend like ok has been called */
-		icon_selected_callback (selection_data->file_entry, selection_data);
+		icon_selected (selection_data);
 	}
 }
 
@@ -811,21 +782,22 @@ eel_gnome_icon_selector_new (const char *title,
 
 	gtk_window_set_resizable (GTK_WINDOW (dialog), TRUE);
 
-	icon_selection = gnome_icon_selection_new();
+	icon_selection = gnome_icon_selection_new ();
 
 	file_entry = gnome_file_entry_new (NULL,NULL);
 	
 	selection_data = g_new0 (IconSelectionData, 1);
+	selection_data->dialog = dialog;
  	selection_data->icon_selection = icon_selection;
  	selection_data->file_entry = file_entry;
  	selection_data->owning_window = owner;
 	selection_data->selection_function = selected;
  	selection_data->callback_data = callback_data;
  	
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox),
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			    file_entry, FALSE, FALSE, 0);
 
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox),
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			    icon_selection, TRUE, TRUE, 0);
 
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
@@ -855,7 +827,7 @@ eel_gnome_icon_selector_new (const char *title,
 				G_CALLBACK (list_icon_selected_callback), selection_data);
 
 	g_signal_connect (entry, "activate",
-			  G_CALLBACK (entry_activated), selection_data);
+			  G_CALLBACK (entry_activated_callback), selection_data);
 
 	return dialog;
 }
@@ -907,36 +879,36 @@ eel_gnome_canvas_item_send_behind (GnomeCanvasItem *item,
 }
 
 ArtIRect
-eel_gnome_canvas_world_to_canvas_window_rectangle (GnomeCanvas *canvas,
-						   ArtDRect world_rect)
+eel_gnome_canvas_world_to_widget_rectangle (GnomeCanvas *canvas,
+					    ArtDRect world_rect)
 {
-	ArtIRect window_rect;
+	ArtIRect widget_rect;
 	
 	g_return_val_if_fail (GNOME_IS_CANVAS (canvas), eel_art_irect_empty);
 
-	eel_gnome_canvas_world_to_canvas_window (canvas,
-						 world_rect.x0, world_rect.y0,
-						 &window_rect.x0, &window_rect.y0);
-	eel_gnome_canvas_world_to_canvas_window (canvas,
-						 world_rect.x1, world_rect.y1,
-						 &window_rect.x1, &window_rect.y1);
+	eel_gnome_canvas_world_to_widget (canvas,
+					  world_rect.x0, world_rect.y0,
+					  &widget_rect.x0, &widget_rect.y0);
+	eel_gnome_canvas_world_to_widget (canvas,
+					  world_rect.x1, world_rect.y1,
+					  &widget_rect.x1, &widget_rect.y1);
 
-	return window_rect;
+	return widget_rect;
 }
 
 void
-eel_gnome_canvas_canvas_window_to_world (GnomeCanvas *canvas,
-					 int          window_x,
-					 int          window_y,
-					 double      *world_x,
-					 double      *world_y)
+eel_gnome_canvas_widget_to_world (GnomeCanvas *canvas,
+				  int          widget_x,
+				  int          widget_y,
+				  double      *world_x,
+				  double      *world_y)
 {
 	int bin_window_x, bin_window_y;
 	
 	g_return_if_fail (GNOME_IS_CANVAS (canvas));
 
-	bin_window_x = window_x + gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (canvas)));
-	bin_window_y = window_y + gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (canvas)));
+	bin_window_x = widget_x + gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (canvas)));
+	bin_window_y = widget_y + gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (canvas)));
 
 	gnome_canvas_window_to_world (canvas,
 				      bin_window_x, bin_window_y,
@@ -944,11 +916,11 @@ eel_gnome_canvas_canvas_window_to_world (GnomeCanvas *canvas,
 }
 
 void
-eel_gnome_canvas_world_to_canvas_window (GnomeCanvas *canvas,
-					 double       world_x,
-					 double       world_y,
-					 int         *window_x,
-					 int         *window_y)
+eel_gnome_canvas_world_to_widget (GnomeCanvas *canvas,
+				  double       world_x,
+				  double       world_y,
+				  int         *widget_x,
+				  int         *widget_y)
 {
 	double bin_window_x, bin_window_y;
 
@@ -958,10 +930,17 @@ eel_gnome_canvas_world_to_canvas_window (GnomeCanvas *canvas,
 				      world_x, world_y,
 				      &bin_window_x, &bin_window_y);
 
-	if (window_x) {
-		*window_x = (int) bin_window_x - gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (canvas)));
+	if (widget_x) {
+		*widget_x = (int) bin_window_x - gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (canvas)));
 	}
-	if (window_y) {
-		*window_y = (int) bin_window_y - gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (canvas)));
+	if (widget_y) {
+		*widget_y = (int) bin_window_y - gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (canvas)));
 	}
 }
+
+char *
+eel_bonobo_make_registration_id (const char *iid)
+{
+	return bonobo_activation_make_registration_id (
+		iid, DisplayString (gdk_display));
+}
